<?php


namespace Drupal\authorization_code\Authentication\Provider;


use Drupal\authorization_code\AuthorizationCodeGenerator;
use Drupal\authorization_code\AuthorizationMethodManager;
use Drupal\Core\Authentication\AuthenticationProviderInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Session\Session;
use Symfony\Component\Serializer\Serializer;

class AuthorizationCode implements AuthenticationProviderInterface, ContainerInjectionInterface {

  /**
   * @var \Symfony\Component\Serializer\Serializer
   */
  protected $serializer;

  /**
   * @var \Symfony\Component\HttpFoundation\Session\Session
   */
  protected $session;

  /**
   * @var \Drupal\authorization_code\AuthorizationMethodManager
   */
  protected $manager;

  /**
   * @var \Drupal\authorization_code\AuthorizationCodeGenerator
   */
  protected $generator;

  /**
   * AuthorizationCode constructor.
   * @param \Symfony\Component\Serializer\Serializer $serializer
   * @param \Symfony\Component\HttpFoundation\Session\Session $session
   * @param \Drupal\authorization_code\AuthorizationMethodManager $manager
   * @param \Drupal\authorization_code\AuthorizationCodeGenerator $generator
   */
  public function __construct(Serializer $serializer, Session $session, AuthorizationMethodManager $manager, AuthorizationCodeGenerator $generator) {
    $this->serializer = $serializer;
    $this->session = $session;
    $this->generator = $generator;
    $this->manager = $manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('serializer'),
      $container->get('session'),
      $container->get('plugin.manager.authorization_method'),
      $container->get('authorization_code.generator')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function applies(Request $request) {
    $session_data = $this->session->get('authorization_code');
    return !empty($session_data['authorized']);
  }

  /**
   * {@inheritdoc}
   */
  public function authenticate(Request $request) {
    if ($session_data = $this->session->get('authorization_code')) {
      return isset($session_data['user']) ? $session_data['user'] : NULL;
    }

    return NULL;
  }

}