<?php

namespace Drupal\authorization_code;

use Drupal\authorization_code\Exception\MissingAuthorizationMethodException;
use Drupal\authorization_code\Plugin\AuthorizationMethodInterface;
use Drupal\authorization_code\Annotation\AuthorizationMethod;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\DefaultPluginManager;

/**
 * Class AuthorizationMethodManager
 * @package Drupal\authorization_code
 */
class AuthorizationMethodManager extends DefaultPluginManager {

  /**
   * @var \Drupal\authorization_code\Plugin\AuthorizationMethodInterface[]
   */
  protected $methods = [];

  /**
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $settings;

  /**
   * AuthorizationMethodManager constructor.
   * @param \Traversable $namespaces
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   */
  public function __construct(\Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler, ImmutableConfig $settings) {
    parent::__construct('Plugin/AuthorizationMethod', $namespaces, $module_handler, AuthorizationMethodInterface::class, AuthorizationMethod::class);
    $this->settings = $settings;
  }

  /**
   * @param array $options
   * @return false|\Drupal\authorization_code\Plugin\AuthorizationMethodInterface
   */
  public function getInstance(array $options) {
    if (isset($options['id'])) {
      $method_id = $options['id'];

      if (empty($this->methods[$method_id])) {
        try {
          $this->methods[$method_id] = $this->createInstance($method_id);
        }
        catch (PluginNotFoundException $e) {
          return FALSE;
        }
      }

      return $this->methods[$method_id];
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  protected function alterDefinitions(&$definitions) {
    $definitions = array_filter($definitions, function($definition) {
      return !$definition['debug_only'] || $this->settings->get('debug_mode_enabled');
    });
    parent::alterDefinitions($definitions);
  }

}
