<?php

namespace Drupal\authorization_code\Plugin;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\TypedData\TypedDataManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

abstract class AuthorizationMethodBase extends PluginBase implements AuthorizationMethodInterface, ContainerFactoryPluginInterface {

  /**
   * @var \Drupal\Core\TypedData\TypedDataManagerInterface
   */
  protected $typedDataManager;

  /**
   * AuthorizationMethodBase constructor.
   * @param array $configuration
   * @param string $plugin_id
   * @param mixed $plugin_definition
   * @param \Drupal\Core\TypedData\TypedDataManagerInterface $typed_data_manager
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, TypedDataManagerInterface $typed_data_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->typedDataManager = $typed_data_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('typed_data_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function isValidArgument($argument) {
    return $this->validateArgument($argument)->count() == 0;
  }

  /**
   * @param mixed $argument
   * @return \Symfony\Component\Validator\ConstraintViolationListInterface
   */
  public function validateArgument($argument) {
    return $this->createArgumentTypedData($argument)->validate();
  }

  /**
   * @param mixed $argument
   * @return \Drupal\Core\TypedData\TypedDataInterface
   */
  protected function createArgumentTypedData($argument) {
    return $this->typedDataManager->create($this->getArgumentType(), $argument);
  }

  /**
   * @return \Drupal\Core\TypedData\DataDefinitionInterface
   */
  protected function getArgumentType() {
    return $this->pluginDefinition['argument_type'];
  }

}