<?php

namespace Drupal\authorization_code\Plugin\rest\resource;

use Drupal\authorization_code\AuthorizationCodeGenerator;
use Drupal\authorization_code\AuthorizationMethodManager;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

/**
 * Provides a resource for database watchdog log entries.
 *
 * @RestResource(
 *   id = "authorization_code:send",
 *   label = @Translation("Send authorization code"),
 *   uri_paths = {
 *     "create" = "/api/authorization-code/send"
 *   }
 * )
 */
class SendCodeResource extends ResourceBase {

  /**
   * @var \Drupal\authorization_code\AuthorizationMethodManager
   */
  protected $manager;

  /**
   * @var \Drupal\authorization_code\AuthorizationCodeGenerator
   */
  protected $generator;

  /**
   * SendCodeResource constructor.
   * @param array $configuration
   * @param string $plugin_id
   * @param mixed $plugin_definition
   * @param array $serializer_formats
   * @param \Psr\Log\LoggerInterface $logger
   * @param \Drupal\authorization_code\AuthorizationMethodManager $manager
   * @param \Drupal\authorization_code\AuthorizationCodeGenerator $generator
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, array $serializer_formats, LoggerInterface $logger, AuthorizationMethodManager $manager, AuthorizationCodeGenerator $generator) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
    $this->generator = $generator;
    $this->manager = $manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('rest'),
      $container->get('plugin.manager.authorization_method'),
      $container->get('authorization_code.generator')
    );
  }

  /**
   * The callback method for this resource.
   *
   * @param array $args
   *   An associative array that must contain the following properties:
   *   - method: string
   *       The plugin id
   *   - argument: mixed
   *       The argument.
   * @return \Drupal\rest\ResourceResponse
   */
  public function post(array $args) {
    $method_id = $args['method'];
    $argument = $args['argument'];
    if (empty($method = $this->manager->getInstance(['id' => $method_id]))) {
      throw new BadRequestHttpException('Invalid method provided');
    }
    if (!$method->isValidArgument($argument)) {
      throw new BadRequestHttpException('Invalid argument provided');
    }

    $key = implode(':', [$method_id, $argument]);
    $method->sendCode($argument, $this->generator->generateCode($key));
    return new ResourceResponse('Code sent');
  }

  /**
   * {@inheritdoc}
   */
  protected function getBaseRouteRequirements($method) {
    return array_merge(parent::getBaseRouteRequirements($method), [
      '_csrf_token' => 'TRUE'
    ]);
  }

}
