<?php

namespace Drupal\authorization_code\Plugin\rest\resource;

use Drupal\Core\Access\CsrfTokenGenerator;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Routing\RouteProviderInterface;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Session\Session;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

/**
 * Provides a resource for database watchdog log entries.
 *
 * @RestResource(
 *   id = "authorization_code:token",
 *   label = @Translation("Authorization code token"),
 *   uri_paths = {
 *     "canonical" = "/api/authorization-code/token/{api_key}"
 *   }
 * )
 */
class TokenResource extends ResourceBase {


  /**
   * @var \Drupal\Core\Access\CsrfTokenGenerator
   */
  protected $tokenGenerator;

  /**
   * @var \Drupal\Core\Routing\RouteProviderInterface
   */
  protected $routeProvider;

  /**
   * @var \Symfony\Component\HttpFoundation\Session\Session
   */
  protected $session;

  /**
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  public function __construct(array $configuration, $plugin_id, $plugin_definition, array $serializer_formats, LoggerInterface $logger, CsrfTokenGenerator $tokenGenerator, RouteProviderInterface $route_provider, Session $session, ImmutableConfig $config) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
    $this->tokenGenerator = $tokenGenerator;
    $this->routeProvider = $route_provider;
    $this->session = $session;
    $this->config = $config;
  }

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('rest'),
      $container->get('csrf_token'),
      $container->get('router.route_provider'),
      $container->get('session'),
      $container->get('config.authorization_code.settings')
    );
  }


  public function get($api_key) {

    // Only allow access to the getToken API if debug mode is enabled.
    if (!$this->config->get('debug_mode_enabled')) {
      throw new BadRequestHttpException('Debug mode is disabled');
    }

    if (!in_array($api_key, ['send', 'authorize'])) {
      throw new BadRequestHttpException('Invalid api key received.');
    }

    $route = $this->routeProvider
      ->getRouteByName("rest.authorization_code.{$api_key}.POST");
    $token = $this->tokenGenerator->get(substr($route->getPath(), 1));
    $this->session->set('authorization_code', ['authorized' => FALSE]);
    $this->session->save();

    return new ResourceResponse([
      'token' => $token
    ]);
  }
}